/**
 * T-Messenger
 * THIS IS NOT FREE SOFTWARE, DO NOT DISTRIBUTE.
 *
 * @summary User Configuration
 * @author Andres Pagella (fokker@alkon.com.ar)
 */

var TM;
if (!TM) TM = {};

if (!TM.settings) TM.settings = {};

TM.settings.info = {
	'file': air.File.applicationStorageDirectory.resolvePath("config.json")
}

/**
 * TM.settings.defaultConfig
 * @summary default user values
 */
TM.settings.defaultConfig = {
    'username': '',
	'password': '',
	'openRepliesNewWindow': true,
	'enableSounds': true,
	'tweetCount': 30,
	'mentionsCount':30,
	'refreshTime': 60,
	'ajaxRequestTimeout': 20,
	'locale': 'en',
	'theme': 'default'
}

/**
 * TM.settings.load
 * @param null
 * @returns boolean, indicating success or error on loading the configuration file
 */
TM.settings.load = function() {	
    if (TM.settings.info.file.exists) {
		var fileHandler = new air.FileStream();
		TM.settings.current = {};
		
        fileHandler.open(TM.settings.info.file, air.FileMode.READ);
        var config = fileHandler.readUTFBytes(TM.settings.info.file.size);

        data = JSON.parse(config);
        for (key in data) TM.settings.current[key] = data[key];

		var username = air.EncryptedLocalStore.getItem('username');
		var password = air.EncryptedLocalStore.getItem('password');
	
		username = username.readUTFBytes(username.bytesAvailable);
		password = password.readUTFBytes(password.bytesAvailable);
	
		TM.settings.current.username = username;
		TM.settings.current.password = password;

		fileHandler.close();
		
		if (TM.settings.current.locale == undefined) {
			alert('Your configuration file needs to be updated.\n\nPlease, restart the application.');
			TM.settings.reset();
			
			return false;
		}

		return true;
    } else {
		TM.settings.current = TM.settings.defaultConfig;

		return false;
	}
};

/**
 * TM.settings.save
 * @param string username (plaintext)
 * @param string password (plaintext)
 * @param object newValues, based on TM.settings.default
 * @returns boolean, indicating success or error on saving the configuration file
 */
TM.settings.save = function(username, password, newValues) {
	try {
		var fileHandler = new air.FileStream();
	
	    fileHandler.open(TM.settings.info.file, air.FileMode.WRITE);
	    fileHandler.writeUTFBytes(JSON.stringify(newValues));
	    TM.settings.current = newValues;

		data = new air.ByteArray();
	    data.writeUTFBytes(username);
	    air.EncryptedLocalStore.setItem('username', data);

	    data = new air.ByteArray();
	    data.writeUTFBytes(password);
	    air.EncryptedLocalStore.setItem('password', data);
	
		fileHandler.close();
		
		return true;
	} catch(e) {
		return false;
	}
};

/**
 * TM.settings.reset
 * @param null
 * @returns boolean, indicating success or error on saving the configuration file
 */
TM.settings.reset = function() {	
    if (TM.settings.info.file.exists) {
		TM.settings.info.file.deleteFile(); // Delete the configuration files.
		
		try {
			TM.db.reset();  // Delete the database.
		} catch(e) { }
		
		air.EncryptedLocalStore.reset(); // Delete both encrypted files for the username/password
		
		air.NativeApplication.nativeApplication.exit(); // Exit the App.

		return true;
	} else {
		return false;
	}
};
